#version 450
#extension GL_EXT_debug_printf : enable
#define POINTLIGHT_COUNT 2
#define DIRLIGHT_COUNT 2
#define SPOTLIGHT_COUNT 1



layout(location = 0) out vec4 color;
//layout(location = 1) in vec3 normal;
layout (location = 0) in vec3 in_normal;
layout (location = 1) in vec3 in_pos;

struct Material
{
	float ka;
	float kd;
	float ks;
	float alpha;
};

struct DirLight
{
	vec3 colVal;
	vec3 dir;
};
struct PointLight
{
	vec3 colVal;
	vec3 pos;
	vec3 atten;
};

struct SpotLight
{
	vec3 col;
	vec3 pos;
	vec3 dir;
	float cutOffOuter;
	float cutOff;
};

layout(binding = 0) uniform CameraBuffer
{
	vec4 colVal;
	vec4 camPos;
	mat4 worldTransform;
	mat4 viewTransform;
	Material m;
}ubo;


int dlc = DIRLIGHT_COUNT;
layout(binding = 1) uniform DL
{
	DirLight[DIRLIGHT_COUNT] dl;
} dirLight;


int plc = POINTLIGHT_COUNT;
layout(binding = 2) uniform PL
{
	PointLight[POINTLIGHT_COUNT] pl;
} pointLight;

int slc = SPOTLIGHT_COUNT;
layout(binding = 3) uniform SL
{
	SpotLight[SPOTLIGHT_COUNT] sl;
} spotLight;

vec3 calc_dir_light_influence(vec3 color,vec3 direction,vec3 V,Material m,vec4 col)
{
	vec3 ldir = normalize(-direction);
	float diff = smoothstep(0.0,1.0,dot(in_normal,ldir));


	vec3 R = reflect(-ldir,in_normal);
	float specInt = pow(smoothstep(0.0,1.0,dot(V,R)),m.alpha);
	vec3 diffuse = color * diff * m.kd;
	vec3 specular =  color * specInt * m.ks;
	return specular + diffuse*col.xyz;
}

vec3 calc_point_light_influence(vec3 color,vec3 pos,vec3 atten,vec3 V,Material m,vec4 col)
{
	vec3 ldir = normalize(pos-in_pos);
	float diff = smoothstep(0.0,1.0,dot(in_normal,ldir));
	vec3 R =
	reflect(-ldir,in_normal);
	float specInt = pow(clamp(dot(V,R),0.0,1.0),m.alpha);
	float dist = length(pos-in_pos);
	float attBetween = atten.x + atten.y * dist + atten.z * (dist * dist);
	float attenuation = 1.0 / attBetween;
	vec3 diffuse = color * diff * m.kd * attenuation;
	vec3 specular = color * specInt * m.ks * attenuation;
	return specular + diffuse*col.xyz;
}

vec3 calc_spot_light_influence(SpotLight sl,vec3 V, Material m, vec4 color)
{
	vec3 sd = normalize(sl.pos-in_pos);
	vec3 ld = normalize(-sl.dir);
	vec3 norm = normalize(in_normal);
	if (dot(sd,ld) > sl.cutOff) {
		float intensity = smoothstep(0.0, 1.0, dot(norm,ld));
		//float mult = lerp(0.0,intensity);
		float spec = 0.0;
		if(intensity > 0.0)
		{
			vec3 eye = normalize(V);
	   	vec3 h = normalize(ld + eye);
	   	float intSpec = clamp(dot(h,norm), 0.0,1.0);
	   	spec = pow(intSpec, m.alpha);
		}


		return (color.xyz*intensity*m.kd+m.ks*spec)/3.0f;
	}
	return vec3(0,0,0);
}

void main() {
	Material m = ubo.m;
	vec3 ia = vec3(1,1,1);

	// Diff light
	vec3 viewDir = normalize(ubo.camPos.xyz-in_pos);
	vec3 ai = m.ka * ia;
	vec3 ip = vec3(0,0,0);
	vec4 col = ubo.colVal;
	for(int i = 0; i < dlc; i++)
	{
		vec3 dirLightInf = calc_dir_light_influence(dirLight.dl[i].colVal,dirLight.dl[i].dir,viewDir,m,col);
		ip += dirLightInf;
	}
	for(int i = 0; i < plc; i++)
	{
		vec3 p2 = calc_point_light_influence(pointLight.pl[i].colVal,pointLight.pl[i].pos,pointLight.pl[i].atten,viewDir,m,col);
		ip += p2;
	}
	for(int i = 0; i < slc; i++)
	{
																		// SpotLight sl,vec3 V, Material m, vec3 color
		vec3 p2 = calc_spot_light_influence(spotLight.sl[i],viewDir,m,col);
		ip += p2;
	}
	//vec3 p3 = calc_spot_light_influence(spotLight.sl[0],viewDir,m,col);
	vec3 outCol = ubo.colVal.xyz * ai + ip;
	color = vec4(outCol,1.0);
}
