#version 450

#define POINTLIGHT_COUNT 2
#define DIRLIGHT_COUNT 2
#define SPOTLIGHT_COUNT 1


layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout (location = 0) out vec4 out_color;
vec3 adapted_normal;
struct Material
{
	float ka;
	float kd;
	float ks;
	float alpha;
};

struct DirLight
{
	vec3 colVal;
	vec3 dir;
};

struct SpotLight
{
	vec3 col;
	vec3 pos;
	vec3 dir;
	float cutOff;
};

struct PointLight
{
	vec3 colVal;
	vec3 pos;
	vec3 atten;
};
layout(binding = 0) uniform CameraBuffer
{
	vec4 colVal;
	vec4 camPos;
	mat4 worldTransform;
	mat4 viewTransform;
	Material m;
	mat4 normTransform;
}ubo;


int dlc = DIRLIGHT_COUNT;
layout(binding = 1) uniform DL
{
	DirLight[DIRLIGHT_COUNT] dl;
} dirLight;


int plc = POINTLIGHT_COUNT;
layout(binding = 2) uniform PL
{
	PointLight[POINTLIGHT_COUNT] pl;
} pointLight;


int slc = SPOTLIGHT_COUNT;
layout(binding = 3) uniform SL
{
	SpotLight[SPOTLIGHT_COUNT] sl;
} spotLight;


vec3 calc_dir_light_influence(vec3 color,vec3 direction,vec3 V,vec3 vertPos,vec4 col)
{
	vec3 ldir = normalize(-direction);
	float diff = smoothstep(0.0,1.0,dot(adapted_normal,ldir));


	vec3 R = reflect(-ldir,adapted_normal);
	float specInt = pow(clamp(dot(V,R),0.0,1.0),ubo.m.alpha);
	vec3 diffuse = color * diff * ubo.m.kd;
	vec3 specular =  color * specInt * ubo.m.ks;
	return diffuse*col.xyz + specular;
}

vec3 calc_point_light_influence(vec3 color,vec3 lpos,vec3 atten,vec3 V,vec3 vertPos,vec4 col)
{
	vec3 ldir = normalize(lpos-vertPos);
	float diff = smoothstep(0.0,1.0,dot(adapted_normal,ldir));
	vec3 R = reflect(-ldir,adapted_normal);
	float specInt = pow(smoothstep(0.0,1.0,dot(V,R)),ubo.m.alpha);

	float dist = length(lpos-vertPos);
	float attBetween = atten.x + atten.y * dist + atten.z * (dist * dist);
	float attenuation = 1.0 / attBetween;
	vec3 diffuse = color * diff * ubo.m.kd * attenuation;
	vec3 specular = color * specInt * ubo.m.ks * attenuation;
	return specular + diffuse*col.xyz;
}

vec3 calc_spot_light_influence(SpotLight sl,vec3 V, Material m, vec4 color,vec3 pos)
{
	vec3 sd = normalize(sl.pos-pos);
	vec3 ld = normalize(-sl.dir);
	vec3 norm = normalize(adapted_normal);
	if (dot(sd,ld) > sl.cutOff) {
		float intensity = smoothstep(0.0, 1.0, dot(norm,ld));
		//float mult = lerp(0.0,intensity);
		float spec = 0.0;
		if(intensity > 0.0)
		{
			vec3 eye = normalize(V);
	   	vec3 h = normalize(ld + eye);
	   	float intSpec = clamp(dot(h,norm), 0.0,1.0);
	   	spec = pow(intSpec, m.alpha);
		}


		return (color.xyz*intensity*m.kd+m.ks*spec)/3.0f;
	}
	return vec3(0,0,0);
}

void main() {

	adapted_normal = (ubo.normTransform * vec4(normal,1.0)).xyz;
	vec3 pos = (ubo.worldTransform*vec4(position, 1.0)).xyz;
	Material m = ubo.m;
	vec3 ia = vec3(1,1,1);

	vec4 col = ubo.colVal;

	// Diff light
	vec3 viewDir = normalize(ubo.camPos.xyz-pos);
	vec3 ai = m.ka * ia;
	vec3 ip = vec3(0,0,0);//
	for(int i = 0; i < dlc; i++)
	{
		vec3 dirLightInf = calc_dir_light_influence(dirLight.dl[i].colVal,dirLight.dl[i].dir,viewDir,pos,col);
		ip += dirLightInf;
	}
	for(int i = 0; i < plc; i++)
	{
		vec3 p2 = calc_point_light_influence(pointLight.pl[i].colVal,pointLight.pl[i].pos,pointLight.pl[i].atten,viewDir,pos,col);
		ip += p2;
	}

	for(int i = 0; i < slc; i++)
	{
		vec3 spotOut = calc_spot_light_influence(spotLight.sl[i],viewDir,m,col,pos);
		ip += spotOut;
	}

	vec3 colorOut = ubo.colVal.xyz * ai + ip;
	out_color = vec4(colorOut,1);



	mat4 transform = ubo.viewTransform * ubo.worldTransform;
    gl_Position = transform*vec4(position, 1.0);
}
